import java.util.Scanner;
import java.util.ArrayList;
import java.util.List;

/* NB: For the judge to run the program, do not modify the declaration of the class Main or
 *     method main() below. The class has to be declared as "class Main { ... }"
 *     and the method as "public static void main(String[] args) { ... }" */
class Main {

	public static void main(String[] args) {

		// Read from the standard input with java.util.Scanner
		Scanner scanner = new Scanner(System.in);

		// Read the number of paragraphs and page width
		int case_no, cases = scanner.nextInt();

		for (case_no = 1; case_no <= cases; case_no++) {

			// Read the number of vertices and edges
			int n = scanner.nextInt();
			int m = scanner.nextInt();
			int i;
			
			// There are several representations of the graph, choose what seems useful:
			
			// For every edge, the source and target vertex  
			int[] edge_from = new int[m];
			int[] edge_to = new int[m];
			
			// For every vertex V, List of vertices X with direct edge from V to X
			List<List<Integer>> v_out = new ArrayList<List<Integer>>();

			// For every vertex V, List of vertices X with direct edge to V from X
			List<List<Integer>> v_in = new ArrayList<List<Integer>>();
			
			// Create the empty lists
			for (i = 0; i < n; i++) {
				v_out.add(new ArrayList<Integer>());
				v_in.add(new ArrayList<Integer>());
			}
			
			// Read elements of A and elements of B
			for (i = 0; i < m; i++) {
				// Read the edge
				int s = scanner.nextInt();
				int t = scanner.nextInt();
				// Insert into all three graph representations
				edge_from[i] = s;
				edge_to[i] = t;
				v_out.get(s).add(t);
				v_in.get(t).add(s);
			}

			// First, compute the topological order
			
			// The List to hold the topological order, empty
			ArrayList<Integer> order = new ArrayList<Integer>();
			
			// The number of *remaining* edges incoming to every vertex
			ArrayList<Integer> in_degree = new ArrayList<Integer>();
			for (i = 0; i < n; i++)
				in_degree.add(v_in.get(i).size());
			
			// The vertices with 0 remaining incoming edges
			ArrayList<Integer> zero_in = new ArrayList<Integer>();
			for (i = 0; i < n; i++) {
				if (in_degree.get(i) == 0)
					zero_in.add(i);
			}
			
			// Now add vertices from zero_in to order one by one,
			// removing their out-edges from their target vertices,
			// update their in_degree and maybe add them to zero_in
			while (!zero_in.isEmpty()) {
				int v = zero_in.remove(zero_in.size() - 1);
				order.add(v);
				for (Integer target: v_out.get(v)) {
					in_degree.set(target, in_degree.get(target) - 1);
					if (in_degree.get(target) == 0)
						zero_in.add(target);
				}
			}
			
			// Now order should contain all vertices -- if not, we have a cycle
			if (order.size() != n)
				System.err.println("Cycles detected in graph!");
			
			// Second, compute the longest path ending at each vertex
			// Length of longest path ending at v
			int[] longest_path = new int[n];
			// Globally maximal length seen
			int max_len = 0;
			
			// Now compute longest_path[v] in order given by "order"
			for (Integer v: order) {
				// Longest path ending at v is at least 1 long
				longest_path[v] = 1;
				
				// All the vertices with edges to v already have the longest path ending there computed
				for (Integer source: v_in.get(v)) {
					// Try if longest path ending at "source" extended to "v" is longer
					// that the longest ending at "v" we already have
					if (longest_path[source] + 1 > longest_path[v])
						longest_path[v] = longest_path[source] + 1;
				}
				
				// Update the global maximum if we found longer
				if (longest_path[v] > max_len) 
					max_len = longest_path[v];
			}
			
			// Output the length of the longest path
			System.out.println(max_len);
		}
		scanner.close();
	}
}
